#include <OutlookEX.au3>
#include <OutlookEX_GUI.au3>
#include <File.au3>

; *****************************************************************************
; Example Script
; Restores previously backed up favorites to the mail navigation folder.
; *****************************************************************************
Global $aGroups[0][0], $iResult, $oOL
Global $sTitle = "Outlook Favorites - Restore"
Global $sBackupFile = @ScriptDir & "\Outlook Favorites - Backup.txt"

; *****************************************************************************
; Connect to Outlook
; *****************************************************************************
$oOL = _OL_Open()
If @error <> 0 Then Exit MsgBox($MB_IconError, $sTitle, "Error connecting to Outlook. @error = " & @error & ", @extended = " & @extended)

; Translate folderpath to the format used by _OL_FolderAccess
_FileReadToArray($sBackupFile, $aGroups, $FRTA_NOCOUNT, "|")
For $i = 0 To UBound($aGroups) - 1
	$aGroups[$i][2] = StringMid($aGroups[$i][2], 3)
Next

; *****************************************************************************
; Add the favorites from the backup file
; *****************************************************************************
$iResult = _OL_NavigationFolderAdd($oOL, $aGroups, Default, $olModuleMail)
If @error <> 0 Then Exit MsgBox($MB_IconError, $sTitle, "Error adding favorites to the navigation folder. @error = " & @error & ", @extended = " & @extended)
MsgBox($MB_IconInformation, $sTitle, UBound($aGroups) & " items restored from " & @CRLF & $sBackupFile & ".", 10)

_OL_Close($oOL)

Func _OL_NavigationFolderAddEX($oOL, $aNavigationFolders, $iStartIndex = Default, $iModuleType = Default)
	Local $oPane, $oModule, $oGroups, $oGroup, $iSet = 0, $oNavigationFolder, $aFolder
	If Not IsObj($oOL) Then Return SetError(1, 0, 0)
	If $iModuleType = Default Then $iModuleType = $olModuleCalendar
	$oPane = $oOL.ActiveExplorer.NavigationPane
	If @error Then Return SetError(2, @error, 0)
	$oModule = $oPane.Modules.GetNavigationModule($iModuleType)
	If @error Then Return SetError(3, @error, 0)
	For $i = $iStartIndex To UBound($aNavigationFolders, 1) - 1
		If StringStripWS($aNavigationFolders[$i][2], $STR_STRIPALL) = "" Then ContinueLoop ; Ignore empty rows
		; Access the folder
		#ignorefunc _OL_FolderAccess
		$aFolder = _OL_FolderAccess($oOL, $aNavigationFolders[$i][2])
		If @error Then Return SetError(6000 + $i, @error, 0)
		$oGroups = $oModule.NavigationGroups
		If @error Then Return SetError(4000 + $i, @error, 0)
		$oGroup = $oGroups.Item($aNavigationFolders[$i][0]) ; Navigation group
		If @error Then Return SetError(5000 + $i, @error, 0)
		$oNavigationFolder = $oGroup.NavigationFolders.Add($aFolder[1])
		If @error Then Return SetError(7000 + $i, @error, 0)
		$oNavigationFolder.DisplayName = $aNavigationFolders[$i][1] ; DisplayName
		$oNavigationFolder.IsSelected = $aNavigationFolders[$i][4] ; If True select the navigation folder for display
		$oNavigationFolder.IsSideBySide = $aNavigationFolders[$i][5] ; If True the navigation folder is displayed in side-by-side else in overlay mode
		$oNavigationFolder.Position = $aNavigationFolders[$i][6] ; Ordinal position of the navigation folder in the navigation pane
		If @error Then Return SetError(8000 + $i, @error, 0)
		$iSet += 1
	Next
	Return $iSet
EndFunc   ;==>_OL_NavigationFolderAdd