#include-once
#include <StringConstants.au3>
#include <MsgBoxConstants.au3>
; #INDEX# =======================================================================================================================
; Title .........: Microsoft Outlook Function Library: Base functions to be used by UDFs OutlookEX (Items) and OutlookEX_GUI (GUI)
; AutoIt Version : See OutlookEX.au3 and OutlookEX_GUI.au3
; UDF Version ...: See variable $__g_sVersionOutlookEX_Base
; Language ......: English
; Description ...: Base functions to access and manipulate Microsoft Outlook using the OutlookEX and OutlookEX_GUI UDF
; Author(s) .....: water
; Modified.......: See variable $__g_sVersionOutlookEX_Base
; Contributors ..: See OutlookEX.au3 and OutlookEX_GUI.au3
; Resources .....: See OutlookEX.au3 and OutlookEX_GUI.au3
; ===============================================================================================================================
Global $__g_sVersionOutlookEX_Base = "OutlookEX_Base: 1.2.3.1 2021-03-10"

#Region #VARIABLES#
; #VARIABLES# ===================================================================================================================
Global $__iOL_Debug = 0 ; Debug level. 0 = no debug information, 1 = Debug info to console, 2 = Debug info to MsgBox, 3 = Debug Info to File
Global $__sOL_DebugFile = @ScriptDir & "\Outlook_Debug.txt" ; Debug file when $__iOL_Debug is set to 3
Global $__oOL_Error ; COM Error handler
Global $__bOL_AlreadyRunning = False ; Is Outlook already running?
; ===============================================================================================================================
#EndRegion #VARIABLES#

#Region #CONSTANTS#
; #CONSTANTS# ===================================================================================================================
; Resources: Outlook 2016 -> Object Model -> Enumerations: https://docs.microsoft.com/de-de/office/vba/api/Outlook

Global Const $olReplyRecipient = 16 ; User defined constant for the reply-recipient type

; MAPI names. Hex values for the Exchange properties that govern aging / archiving
; See: http://technet.microsoft.com/en-us/query/cc815492
Global Const $iPR_AGING_AGE_FOLDER = 0x6857000B ; BOOL Enable aging aka Archive for this folder: True = Enabled False = Disabled
Global Const $iPR_AGING_GRANULARITY = 0x36EE0003 ; LONG Aging granularity: 0 = Months 1 = Weeks 2 = Days
Global Const $iPR_AGING_PERIOD = 0x36EC0003 ; LONG, duration from 1 to 999 (combined with AGING GRANULARITY)
Global Const $iPR_AGING_DELETE_ITEMS = 0x6855000B ; BOOL FALSE = archive, TRUE = permanently delete
Global Const $iPR_AGING_FILE_NAME_AFTER9 = 0x6859001E ; STRING Path and filename of archive file for Exchange version > Exchange 9
Global Const $iPR_AGING_DEFAULT = 0x685E0003 ; LONG values unclear, seems like 3=do not archive, 1=archive according to defaults, 0=custom settings

; MsoAppLanguageID Enumeration. Specifies a language setting in a Microsoft Office application
; See: http://msdn.microsoft.com/en-us/library/aa432459%28v=office.12%29.aspx
Global Const $msoLanguageIDExeMode = 4 ; Execution mode language
Global Const $msoLanguageIDHelp = 3 ; Help language
Global Const $msoLanguageIDInstall = 1 ; Install language
Global Const $msoLanguageIDUI = 2 ; User interface language
Global Const $msoLanguageIDUIPrevious = 5 ; User interface language used prior to the current user interface language

; OlAddressEntryUserType Enumeration. Represents the type of user for the AddressEntry or object derived from AddressEntry.
Global Const $olExchangeAgentAddressEntry = 3 ; An address entry that is an Exchange agent
Global Const $olExchangeDistributionListAddressEntry = 1 ; An address entry that is an Exchange distribution list
Global Const $olExchangeOrganizationAddressEntry = 4 ; An address entry that is an Exchange organization
Global Const $olExchangePublicFolderAddressEntry = 2 ; An address entry that is an Exchange public folder
Global Const $olExchangeRemoteUserAddressEntry = 5 ; An Exchange user that belongs to a different Exchange forest
Global Const $olExchangeUserAddressEntry = 0 ; An Exchange user that belongs to the same Exchange forest
Global Const $olLdapAddressEntry = 20 ; An address entry that uses the Lightweight Directory Access Protocol (LDAP)
Global Const $olOtherAddressEntry = 40 ; A custom or some other type of address entry such as FAX
Global Const $olOutlookContactAddressEntry = 10 ; An address entry in an Outlook Contacts folder
Global Const $olOutlookDistributionListAddressEntry = 11 ; An address entry that is an Outlook distribution list
Global Const $olSmtpAddressEntry = 30 ; An address entry that uses the Simple Mail Transfer Protocol (SMTP)

; OlAddressListType Enumeration. Represents the type of AddressList.
Global Const $olCustomAddressList = 4 ; A custom address book provider
Global Const $olExchangeContainer = 1 ; A container for address lists on an Exchange server
Global Const $olExchangeGlobalAddressList = 0 ; An Exchange Global Address List
Global Const $olOutlookAddressList = 2 ; An address list that corresponds to the Outlook Contacts Address Book
Global Const $olOutlookLdapAddressList = 3 ; An address list that uses the Lightweight Directory Access Protocol (LDAP)

; OlAlign Enumeration. Specifies the alignment for the Label property of the ColumnFormat object.
Global Const $olAlignCenter = 1 ; Indicates that the label for the specified column should be centered.
Global Const $olAlignLeft = 0 ; Indicates that the label for the specified column should be left-aligned.
Global Const $olAlignRight = 2 ; Indicates that the label for the specified column should be right-aligned.

; OlAttachmentType Enumeration. Specifies the attachment type.
Global Const $olByReference = 4 ; The attachment is a shortcut to the location of the original file
Global Const $olByValue = 1 ; The attachment is a copy of the original file and can be accessed even if the original file is removed
Global Const $olEmbeddeditem = 5 ; The attachment is an Outlook message format file (.msg) and is a copy of the original message
Global Const $olOLE = 6 ; The attachment is an OLE document

; OlBodyFormat Enumeration. Specifies the format of the body text of an item.
Global Const $olFormatHTML = 2 ; HTML format
Global Const $olFormatPlain = 1 ; Plain format
Global Const $olFormatRichText = 3 ; Rich text format
Global Const $olFormatUnspecified = 0 ; Unspecified format

; OlBusyStatus Enumeration. Indicates a user's availability.
Global Const $olBusy = 2 ; The user is busy
Global Const $olFree = 0 ; The user is available
Global Const $olOutOfOffice = 3 ; The user is out of office
Global Const $olTentative = 1 ; The user has a tentative appointment scheduled

; OlCategoryColor Enumeration. Indicates the color specified in the Color property of the Category object.
Global Const $olCategoryColorBlack = 15 ; Black
Global Const $olCategoryColorBlue = 8 ; Blue
Global Const $olCategoryColorDarkBlue = 23 ; Dark Blue
Global Const $olCategoryColorDarkGray = 14 ; Dark Gray
Global Const $olCategoryColorDarkGreen = 20 ; Dark Green
Global Const $olCategoryColorDarkMaroon = 25 ; Dark Maroon
Global Const $olCategoryColorDarkOlive = 22 ; Dark Olive
Global Const $olCategoryColorDarkOrange = 17 ; Dark Orange
Global Const $olCategoryColorDarkPeach = 18 ; Dark Peach
Global Const $olCategoryColorDarkPurple = 24 ; Dark Purple
Global Const $olCategoryColorDarkRed = 16 ; Dark Red
Global Const $olCategoryColorDarkSteel = 12 ; Dark Steel
Global Const $olCategoryColorDarkTeal = 21 ; Dark Teal
Global Const $olCategoryColorDarkYellow = 19 ; Dark Yellow
Global Const $olCategoryColorGray = 13 ; Gray
Global Const $olCategoryColorGreen = 5 ; Green
Global Const $olCategoryColorMaroon = 10 ; Maroon
Global Const $olCategoryColorNone = -1 ; No color assigned
Global Const $olCategoryColorOlive = 7 ; Olive
Global Const $olCategoryColorOrange = 2 ; Orange
Global Const $olCategoryColorPeach = 3 ; Peach
Global Const $olCategoryColorPurple = 9 ; Purple
Global Const $olCategoryColorRed = 1 ; Red
Global Const $olCategoryColorSteel = 11 ; Steel
Global Const $olCategoryColorTeal = 6 ; Teal
Global Const $olCategoryColorYellow = 4 ; Yellow

; OlCategoryShortcutKey Enumeration. Identifies the shortcut key specified for the ShortcutKey property of the Category object.
Global Const $olCategoryShortcutKeyCtrlF10 = 10 ; CTRL+F10
Global Const $olCategoryShortcutKeyCtrlF11 = 11 ; CTRL+F11
Global Const $olCategoryShortcutKeyCtrlF12 = 12 ; CTRL+F12
Global Const $olCategoryShortcutKeyCtrlF2 = 2 ; CTRL+F2
Global Const $olCategoryShortcutKeyCtrlF3 = 3 ; CTRL+F3
Global Const $olCategoryShortcutKeyCtrlF4 = 4 ; CTRL+F4
Global Const $olCategoryShortcutKeyCtrlF5 = 5 ; CTRL+F5
Global Const $olCategoryShortcutKeyCtrlF6 = 6 ; CTRL+F6
Global Const $olCategoryShortcutKeyCtrlF7 = 7 ; CTRL+F7
Global Const $olCategoryShortcutKeyCtrlF8 = 8 ; CTRL+F8
Global Const $olCategoryShortcutKeyCtrlF9 = 9 ; CTRL+F9
Global Const $olCategoryShortcutKeyNone = 0 ; No shortcut key specified

; OlDaysOfWeek Enumeration. Constants representing days of the week.
Global Const $olFriday = 32 ; Friday
Global Const $olMonday = 2 ; Monday
Global Const $olSaturday = 64 ; Saturday
Global Const $olSunday = 1 ; Sunday
Global Const $olThursday = 16 ; Thursday
Global Const $olTuesday = 4 ; Tuesday
Global Const $olWednesday = 8 ; Wednesday

; OlDefaultFolders Enumeration. Specifies the folder type for the current Outlook profile.
Global Const $olFolderCalendar = 9 ; The Calendar folder
Global Const $olFolderConflicts = 19 ; The Conflicts folder (subfolder of the Sync Issues folder) Only available for an Exchange account
Global Const $olFolderContacts = 10 ; The Contacts folder
Global Const $olFolderDeletedItems = 3 ; The Deleted Items folder
Global Const $olFolderDrafts = 16 ; The Drafts folder
Global Const $olFolderInbox = 6 ; The Inbox folder
Global Const $olFolderJournal = 11 ; The Journal folder
Global Const $olFolderJunk = 23 ; The Junk E-Mail folder
Global Const $olFolderLocalFailures = 21 ; The Local Failures folder (subfolder of the Sync Issues folder) Only available for an Exchange account
Global Const $olFolderManagedEmail = 29 ; The top-level folder in the Managed Folders group For more information on Managed Folders, see the Help in Microsoft Outlook Only available for an Exchange account
Global Const $olFolderNotes = 12 ; The Notes folder
Global Const $olFolderOutbox = 4 ; The Outbox folder
Global Const $olFolderSentMail = 5 ; The Sent Mail folder
Global Const $olFolderServerFailures = 22 ; The Server Failures folder (subfolder of the Sync Issues folder) Only available for an Exchange account
Global Const $olFolderSuggestedContacts = 30 ; The Suggested Contacts folder
Global Const $olFolderSyncIssues = 20 ; The Sync Issues folder Only available for an Exchange account
Global Const $olFolderTasks = 13 ; The Tasks folder
Global Const $olFolderToDo = 28 ; The To Do folder
Global Const $olPublicFoldersAllPublicFolders = 18 ; The All Public Folders folder in the Exchange Public Folders store Only available for an Exchange account
Global Const $olFolderRssFeeds = 25 ; The RSS Feeds folder

; OlDefaultSelectNamesDisplayMode Enumeration. Specifies the default caption, the number of buttons, the button labels, and the address lists to display in the Select Names dialog box.
Global Const $olDefaultDelegates = 6
Global Const $olDefaultMail = 1
Global Const $olDefaultMeeting = 2
Global Const $olDefaultMembers = 5
Global Const $olDefaultPickRooms = 8
Global Const $olDefaultSharingRequest = 4
Global Const $olDefaultSingleName = 7
Global Const $olDefaultTask = 3

; OlExchangeStoreType Enumeration. Specifies the type of Exchange store.
Global Const $olExchangeMailbox = 1 ; Specifies an Exchange delegate mailbox store
Global Const $olExchangePublicFolder = 2 ; Specifies an Exchange Public Folder store
Global Const $olNotExchange = 3 ; Specifies that the store is not an Exchange store
Global Const $olAdditionalExchangeMailbox = 4 ; Specifies an additional Exchange mailbox store
Global Const $olPrimaryExchangeMailbox = 0 ; Specifies a primary Exchange mailbox store

; OlImportance Enumeration. Specifies the level of importance for an item marked by the creator of the item.
Global Const $olImportanceHigh = 2 ; Item is marked as high importance
Global Const $olImportanceLow = 0 ; Item is marked as low importance
Global Const $olImportanceNormal = 1 ; Item is marked as medium importance

; OlInspectorClose Enumeration. Indicates the save mode.
Global Const $olDiscard = 1 ; Changes to the document are discarded
Global Const $olPromptForSave = 2 ; User is prompted to save documents
Global Const $olSave = 0 ; Documents are saved

; OlItemType Enumeration. Indicates the Outlook Item type.
Global Const $olAppointmentItem = 1 ; Represents an AppointmentItem
Global Const $olContactItem = 2 ; Represents a ContactItem
Global Const $olDistributionListItem = 7 ; Represents an DistListItem
Global Const $olJournalItem = 4 ; Represents a JournalItem
Global Const $olMailItem = 0 ; Represents a MailItem
Global Const $olNoteItem = 5 ; Represents a NoteItem
Global Const $olPostItem = 6 ; Represents a PostItem
Global Const $olTaskItem = 3 ; Represents a TaskItem

; OlMailRecipientType Enumeration. Indicates the recipient type for the Item.
Global Const $olBCC = 3 ; The recipient is specified in the BCC property of the Item
Global Const $olCC = 2 ; The recipient is specified in the CC property of the Item
Global Const $olOriginator = 0 ; Originator (sender) of the Item
Global Const $olTo = 1 ; The recipient is specified in the To property of the Item

; OlMarkInterval Enumeration. Specifies the time period for which an Outlook item is marked as a task.
Global Const $olMarkComplete = 5 ; Mark the task as complete
Global Const $olMarkNoDate = 4 ; Mark the task due with no date
Global Const $olMarkNextWeek = 3 ; Mark the task due next week
Global Const $olMarkThisWeek = 2 ; Mark the task due this week
Global Const $olMarkToday = 0 ; Mark the task due today
Global Const $olMarkTomorrow = 1 ; Mark the task due tomorrow

; OlMeetingRecipientType Enumeration. Indicates the recipient type for the meeting.
Global Const $olOptional = 2 ; Optional attendee
Global Const $olOrganizer = 0 ; Meeting organizer
Global Const $olRequired = 1 ; Required attendee
Global Const $olResource = 3 ; A resource such as a conference room

; OlMeetingResponse Enumeration. Indicates the response to a meeting request.
Global Const $olMeetingAccepted = 3 ; The meeting was accepted
Global Const $olMeetingDeclined = 4 ; The meeting was declined
Global Const $olMeetingTentative = 2 ; The meeting was tentatively accepted

; OlMeetingStatus Enumeration. Indicates the status of the meeting.
Global Const $olMeeting = 1 ; The meeting has been scheduled
Global Const $olMeetingCanceled = 5 ; The scheduled meeting has been cancelled
Global Const $olMeetingReceived = 3 ; The meeting request has been received
Global Const $olMeetingReceivedAndCanceled = 7 ; The scheduled meeting has been cancelled but still appears on the user's calendar
Global Const $olNonMeeting = 0 ; An Appointment item without attendees has been scheduled. This status can be used to set up holidays on a calendar

; OlNavigationModuleType Enumeration. Identifies the navigation module type of a NavigationModule object.
Global Const $olModuleCalendar = 1 ; Identifies a CalendarModule object that represents the Calendar navigation module
Global Const $olModuleContacts = 2 ; Identifies a ContactsModule object that represents the Contacts navigation module
Global Const $olModuleFolderList = 6 ; Identifies a NavigationModule object that represents the Folders List navigation module
Global Const $olModuleJournal = 4 ; Identifies a JournalModule object that represents the Journal navigation module
Global Const $olModuleMail = 0 ; Identifies a MailModule object that represents the Mail navigation module
Global Const $olModuleNotes = 5 ; Identifies a NotesModule object that represents the Notes navigation module
Global Const $olModuleShortcuts = 7 ; Identifies a NavigationModule object that represents the Shortcuts navigation module
Global Const $olModuleTasks = 3 ; Identifies a TasksModule object that represents the Tasks navigation module

; OlObjectClass Enumeration. Constants representing the different Microsoft Office Outlook object classes.
Global Const $olAccount = 105 ; Represents an Account object
Global Const $olAccountRuleCondition = 135 ; Represents an AccountRuleCondition object
Global Const $olAccounts = 106 ; Represents an Accounts object
Global Const $olAction = 32 ; Represents an Action object
Global Const $olActions = 33 ; Represents an Actions object
Global Const $olAddressEntries = 21 ; Represents an AddressEntries object
Global Const $olAddressEntry = 8 ; Represents an AddressEntry object
Global Const $olAddressList = 7 ; Represents an AddressList object
Global Const $olAddressLists = 20 ; Represents an AddressLists object
Global Const $olAddressRuleCondition = 170 ; Represents an AddressRuleCondition object
Global Const $olApplication = 0 ; Represents an Application object
Global Const $olAppointment = 26 ; Represents an AppointmentItem object
Global Const $olAssignToCategoryRuleAction = 122 ; Represents an AssignToCategoryRuleAction object
Global Const $olAttachment = 5 ; Represents an Attachment object
Global Const $olAttachments = 18 ; Represents an Attachments object
Global Const $olAttachmentSelection = 169 ; Represents an AttachmentSelection object
Global Const $olAutoFormatRule = 147 ; Represents an AutoFormatRule object
Global Const $olAutoFormatRules = 148 ; Represents an AutoFormatRules object
Global Const $olCalendarModule = 159 ; Represents a CalendarModule object
Global Const $olCalendarSharing = 151 ; Represents a CalendarSharing object
Global Const $olCategories = 153 ; Represents a Categories object
Global Const $olCategory = 152 ; Represents a Category object
Global Const $olCategoryRuleCondition = 130 ; Represents a CategoryRuleCondition object
Global Const $olClassBusinessCardView = 168 ; Represents a BusinessCardView object
Global Const $olClassCalendarView = 139 ; Represents a CalendarView object
Global Const $olClassCardView = 138 ; Represents a CardView object
Global Const $olClassIconView = 137 ; Represents a IconView object
Global Const $olClassNavigationPane = 155 ; Represents a NavigationPane object
Global Const $olClassTableView = 136 ; Represents a TableView object
Global Const $olClassTimeLineView = 140 ; Represents a TimelineView object
Global Const $olClassTimeZone = 174 ; Represents a TimeZone object
Global Const $olClassTimeZones = 175 ; Represents a TimeZones object
Global Const $olColumn = 154 ; Represents a Column object
Global Const $olColumnFormat = 149 ; Represents a ColumnFormat object
Global Const $olColumns = 150 ; Represents a Columns object
Global Const $olConflict = 102 ; Represents a Conflict object
Global Const $olConflicts = 103 ; Represents a Conflicts object
Global Const $olContact = 40 ; Represents a ContactItem object
Global Const $olContactsModule = 160 ; Represents a ContactsModule object
Global Const $olDistributionList = 69 ; Represents a ExchangeDistributionList object
Global Const $olDocument = 41 ; Represents a DocumentItem object
Global Const $olException = 30 ; Represents an Exception object
Global Const $olExceptions = 29 ; Represents an Exceptions object
Global Const $olExchangeDistributionList = 111 ; Represents an ExchangeDistributionList object
Global Const $olExchangeUser = 110 ; Represents an ExchangeUser object
Global Const $olExplorer = 34 ; Represents an Explorer object
Global Const $olExplorers = 60 ; Represents an Explorers object
Global Const $olFolder = 2 ; Represents a Folder object
Global Const $olFolders = 15 ; Represents a Folders object
Global Const $olFolderUserProperties = 172 ; Represents a UserDefinedProperties object
Global Const $olFolderUserProperty = 171 ; Represents a UserDefinedProperty object
Global Const $olFormDescription = 37 ; Represents a FormDescription object
Global Const $olFormNameRuleCondition = 131 ; Represents a FormNameRuleCondition object
Global Const $olFormRegion = 129 ; Represents a FormRegion object
Global Const $olFromRssFeedRuleCondition = 173 ; Represents a FromRssFeedRuleCondition object
Global Const $olFromRuleCondition = 132 ; Represents a ToOrFromRuleCondition object
Global Const $olImportanceRuleCondition = 128 ; Represents an ImportanceRuleCondition object
Global Const $olInspector = 35 ; Represents an Inspector object
Global Const $olInspectors = 61 ; Represents an Inspectors object
Global Const $olItemProperties = 98 ; Represents an ItemProperties object
Global Const $olItemProperty = 99 ; Represents an ItemProperty object
Global Const $olItems = 16 ; Represents an Items object
Global Const $olJournal = 42 ; Represents a JournalItem object
Global Const $olJournalModule = 162 ; Represents a JournalModule object
Global Const $olLink = 75 ; Represents a Link object
Global Const $olLinks = 76 ; Represents a Links object
Global Const $olMail = 43 ; Represents a MailItem object
Global Const $olMailModule = 158 ; Represents a MailModule object
Global Const $olMarkAsTaskRuleAction = 124 ; Represents a MarkAsTaskRuleAction object
Global Const $olMeetingCancellation = 54 ; Represents a MeetingItem object that is a meeting cancellation notice
Global Const $olMeetingRequest = 53 ; Represents a MeetingItem object that is a meeting request
Global Const $olMeetingResponseNegative = 55 ; Represents a MeetingItem object that is a refusal of a meeting request
Global Const $olMeetingResponsePositive = 56 ; Represents a MeetingItem object that is an acceptance of a meeting request
Global Const $olMeetingResponseTentative = 57 ; Represents a MeetingItem object that is a tentative acceptance of a meeting request
Global Const $olMoveOrCopyRuleAction = 118 ; Represents a MoveOrCopyRuleAction object
Global Const $olNamespace = 1 ; Represents a NameSpace object
Global Const $olNavigationFolder = 167 ; Represents a NavigationFolder object
Global Const $olNavigationFolders = 166 ; Represents a NavigationFolders object
Global Const $olNavigationGroup = 165 ; Represents a NavigationGroup object
Global Const $olNavigationGroups = 164 ; Represents a NavigationGroups object
Global Const $olNavigationModule = 157 ; Represents a NavigationModule object
Global Const $olNavigationModules = 156 ; Represents a NavigationModules object
Global Const $olNewItemAlertRuleAction = 125 ; Represents a NewItemAlertRuleAction object
Global Const $olNote = 44 ; Represents a NoteItem object
Global Const $olNotesModule = 163 ; Represents a NotesModule object
Global Const $olOrderField = 144 ; Represents an OrderField object
Global Const $olOrderFields = 145 ; Represents an OrderFields object
Global Const $olOutlookBarGroup = 66 ; Represents an OutlookBarGroup object
Global Const $olOutlookBarGroups = 65 ; Represents an OutlookBarGroups object
Global Const $olOutlookBarPane = 63 ; Represents an OutlookBarPane object
Global Const $olOutlookBarShortcut = 68 ; Represents an OutlookBarShortcut object
Global Const $olOutlookBarShortcuts = 67 ; Represents an OutlookBarShortcuts object
Global Const $olOutlookBarStorage = 64 ; Represents an OutlookBarStorage object
Global Const $olPages = 36 ; Represents a Pages object
Global Const $olPanes = 62 ; Represents a Panes object
Global Const $olPlaySoundRuleAction = 123 ; Represents a PlaySoundRuleAction object
Global Const $olPost = 45 ; Represents a PostItem object
Global Const $olPropertyAccessor = 112 ; Represents a PropertyAccessor object
Global Const $olPropertyPages = 71 ; Represents a PropertyPages object
Global Const $olPropertyPageSite = 70 ; Represents a PropertyPageSite object
Global Const $olRecipient = 4 ; Represents a Recipient object
Global Const $olRecipients = 17 ; Represents a Recipients object
Global Const $olRecurrencePattern = 28 ; Represents a RecurrencePattern object
Global Const $olReminder = 101 ; Represents a Reminder object
Global Const $olReminders = 100 ; Represents a Reminders object
Global Const $olRemote = 47 ; Represents a RemoteItem object
Global Const $olReport = 46 ; Represents a ReportItem object
Global Const $olResults = 78 ; Represents a Results object
Global Const $olRow = 121 ; Represents a Row object
Global Const $olRule = 115 ; Represents a Rule object
Global Const $olRuleAction = 117 ; Represents a RuleAction object
Global Const $olRuleActions = 116 ; Represents a RuleAction object
Global Const $olRuleCondition = 127 ; Represents a RuleCondition object
Global Const $olRuleConditions = 126 ; Represents a RuleConditions object
Global Const $olRules = 114 ; Represents a Rules object
Global Const $olSearch = 77 ; Represents a Search object
Global Const $olSelection = 74 ; Represents a Selection object
Global Const $olSelectNamesDialog = 109 ; Represents a SelectNamesDialog object
Global Const $olSenderInAddressListRuleCondition = 133 ; Represents a SenderInAddressListRuleCondition object
Global Const $olSendRuleAction = 119 ; Represents a SendRuleAction object
Global Const $olSharing = 104 ; Represents a SharingItem object
Global Const $olStorageItem = 113 ; Represents a StorageItem object
Global Const $olStore = 107 ; Represents a Store object
Global Const $olStores = 108 ; Represents a Stores object
Global Const $olSyncObject = 72 ; Represents a SyncObject object
Global Const $olSyncObjects = 73 ; Represents a SyncObject object
Global Const $olTable = 120 ; Represents a Table object
Global Const $olTask = 48 ; Represents a TaskItem object
Global Const $olTaskRequest = 49 ; Represents a TaskRequestItem object
Global Const $olTaskRequestAccept = 51 ; Represents a TaskRequestAcceptItem object
Global Const $olTaskRequestDecline = 52 ; Represents a TaskRequestDeclineItem object
Global Const $olTaskRequestUpdate = 50 ; Represents a TaskRequestUpdateItem object
Global Const $olTasksModule = 161 ; Represents a TasksModule object
Global Const $olTextRuleCondition = 134 ; Represents a TextRuleCondition object
Global Const $olUserDefinedProperties = 172 ; Represents a UserDefinedProperties object
Global Const $olUserDefinedProperty = 171 ; Represents a UserDefinedProperty object
Global Const $olUserProperties = 38 ; Represents a UserProperties object
Global Const $olUserProperty = 39 ; Represents a UserProperty object
Global Const $olView = 80 ; Represents a View object
Global Const $olViewField = 142 ; Represents a ViewField object
Global Const $olViewFields = 141 ; Represents a ViewFields object
Global Const $olViewFont = 146 ; Represents a ViewFont object
Global Const $olViews = 79 ; Represents a Views object

; OlPane Enumeration. Specifies the pane (window).
Global Const $olFolderList = 2 ; Folder list pane
Global Const $olNavigationPane = 4 ; Navigation pane
Global Const $olOutlookBar = 1 ; Outlook bar pane
Global Const $olPreview = 3 ; Preview pane
Global Const $olToDoBar = 5 ; To Do bar pane

; OlRecurrenceType Enumeration. Specifies the recurrence pattern type.
Global Const $olRecursDaily = 0 ; Represents a daily recurrence pattern
Global Const $olRecursMonthly = 2 ; Represents a monthly recurrence pattern
Global Const $olRecursMonthNth = 3 ; Represents a MonthNth recurrence pattern. See RecurrencePattern.Instance property
Global Const $olRecursWeekly = 1 ; Represents a weekly recurrence pattern
Global Const $olRecursYearly = 5 ; Represents a yearly recurrence pattern
Global Const $olRecursYearNth = 6 ; Represents a YearNth recurrence pattern. See RecurrencePattern.Instance property

; OlRuleActionType Enumeration. Specifies the type of rule action for a rule.
Global Const $olRuleActionAssignToCategory = 2 ; Rule action is to assign categories to the message
Global Const $olRuleActionCcMessage = 27 ; Rule action is to cc the message to specified recipients
Global Const $olRuleActionClearCategories = 30 ; Rule action is to clear all the categories assigned to the message
Global Const $olRuleActionCopyToFolder = 5 ; Rule action is to copy the message to a specified folder
Global Const $olRuleActionCustomAction = 22 ; Rule action is to perform a custom action
Global Const $olRuleActionDefer = 28 ; Rule action is to defer delivery of the message by the specified number of minutes
Global Const $olRuleActionDelete = 3 ; Rule action is to delete the message
Global Const $olRuleActionDeletePermanently = 4 ; Rule action is to permanently delete the message
Global Const $olRuleActionDesktopAlert = 24 ; Rule action is to display a desktop alert
Global Const $olRuleActionFlagClear = 13 ; Rule action is to clear the message flag
Global Const $olRuleActionFlagColor = 12 ; Rule action is to flag the message with a specified colored flag
Global Const $olRuleActionFlagForActionInDays = 11 ; Rule action is to flag the message for action in the specified number of days
Global Const $olRuleActionForward = 6 ; Rule action is to forward the message to the specified recipients
Global Const $olRuleActionForwardAsAttachment = 7 ; Rule action is to forward the message as an attachment to the specified recipients
Global Const $olRuleActionImportance = 14 ; Rule action is to mark the message with the specified level of importance
Global Const $olRuleActionMarkAsTask = 29 ; Rule action is to mark the message as a task (Bug in the docu: docu says 41. Wrong hex to dec conversion)
Global Const $olRuleActionMarkRead = 19 ; Rule action is to mark the message as read
Global Const $olRuleActionMoveToFolder = 1 ; Rule action is to move the message to the specified folder
Global Const $olRuleActionNewItemAlert = 23 ; Rule action is to display the specified text in the New Item Alert dialog box
Global Const $olRuleActionNotifyDelivery = 26 ; Rule action is to request delivery notification for the message being sent
Global Const $olRuleActionNotifyRead = 25 ; Rule action is to request read notification for the message being sent
Global Const $olRuleActionPlaySound = 17 ; Rule action is to play a sound file
Global Const $olRuleActionPrint = 16 ; Rule action is to print the message on the default printer
Global Const $olRuleActionRedirect = 8 ; Rule action is to redirect the message to the specified recipients
Global Const $olRuleActionRunScript = 20 ; Rule action is to run a script
Global Const $olRuleActionSensitivity = 15 ; Rule action is to mark the message with the specified level of sensitivity
Global Const $olRuleActionServerReply = 9 ; Rule action is to request the server to reply with the specified mail item
Global Const $olRuleActionStartApplication = 18 ; Rule action is to run an .exe file
Global Const $olRuleActionStop = 21 ; Rule action is to stop processing more rules
Global Const $olRuleActionTemplate = 10 ; Rule action is to use the specified template (.oft) file as a form template
Global Const $olRuleActionUnknown = 0 ; Unrecognized rule action

; OlRuleConditionType Enumeration. Specifies the type of rule condition or exception condition of a rule.
Global Const $olConditionAccount = 3 ; Account is the account specified in AccountRuleCondition.Account
Global Const $olConditionAnyCategory = 29 ; Message is assigned to any category
Global Const $olConditionBody = 13 ; Body contains words specified in TextRuleCondition.Text
Global Const $olConditionBodyOrSubject = 14 ; Body or subject contains words specified by TextRuleCondition.Text
Global Const $olConditionCategory = 18 ; Category is the category specified in CategoryRuleCondition.Categories
Global Const $olConditionCc = 9 ; Message has my name in the Cc box
Global Const $olConditionDateRange = 22 ; Message was received between x and y, where x and y are Date values
Global Const $olConditionFlaggedForAction = 8 ; Message is flagged for the specified action
Global Const $olConditionFormName = 23 ; Message uses the form specified in FormNameRuleCondition.FormName
Global Const $olConditionFrom = 1 ; Sender is in the recipient list specified in ToOrFromRuleCondition.Recipients
Global Const $olConditionFromAnyRssFeed = 31 ; Message is generated from any RSS subscription
Global Const $olConditionFromRssFeed = 30 ; Message is generated from a specific RSS subscription
Global Const $olConditionHasAttachment = 20 ; Message has one or more attachments
Global Const $olConditionImportance = 6 ; Message is marked with the specified level of importance
Global Const $olConditionLocalMachineOnly = 27 ; Rule can run only on the local machine
Global Const $olConditionMeetingInviteOrUpdate = 26 ; Message is a meeting invitation or update
Global Const $olConditionMessageHeader = 15 ; Message header contains words specified in TextRuleCondition.Text
Global Const $olConditionNotTo = 11 ; Message does not have my name in the To box
Global Const $olConditionOnlyToMe = 4 ; Message is sent only to me
Global Const $olConditionOOF = 19 ; Message is an out-of-office message
Global Const $olConditionOtherMachine = 28 ; Rule can run only on a specific machine that is not the current machine
Global Const $olConditionProperty = 24 ; Document property is exactly, contains, or does not contain specified properties
Global Const $olConditionRecipientAddress = 16 ; Recipient address contains words specified in TextRuleCondition.Text
Global Const $olConditionSenderAddress = 17 ; Sender address contains words specified in TextRuleCondition.Text
Global Const $olConditionSenderInAddressBook = 25 ; Sender is in the address list specified in AddressRuleCondition.Address
Global Const $olConditionSensitivity = 7 ; Message is marked with the specified level of sensitivity
Global Const $olConditionSentTo = 12 ; Sent to recipients (To, Cc) are in the recipient list specified in ToOrFromRuleCondition.Recipients
Global Const $olConditionSizeRange = 21 ; Message size is between x and y in units of KB, where x and y are Integer values
Global Const $olConditionSubject = 2 ; Subject contains words specified in TextRuleCondition.Text
Global Const $olConditionTo = 5 ; My name is in the To box
Global Const $olConditionToOrCc = 10 ; Message has my name in the To or Cc box
Global Const $olConditionUnknown = 0 ; Unrecognized condition

; OlRuleExecuteOption Enumeration. Specifies the type of messages in the specified folder or folders that a rule should be applied to.
Global Const $olRuleExecuteAllMessages = 0 ; Executes a rule against all messages in the specified folder or folders
Global Const $olRuleExecuteReadMessages = 1 ; Executes a rule against messages that have been read in the specified folder or folders
Global Const $olRuleExecuteUnreadMessages = 2 ; Executes a rule against messages that have not been read in the specified folder or folders

; OlRuleType Enumeration. Indicates if a rule is applied to messages that are being sent or received.
Global Const $olRuleReceive = 0 ; Indicates that the rule is applied to messages that are being received
Global Const $olRuleSend = 1 ; Indicates that the rule is being applied to messages being sent

; OlSaveAsType Enumeration. Specifies the file format.
Global Const $olDoc = 4 ; Microsoft Office Word format (.doc)
Global Const $olHTML = 5 ; HTML format (.html)
Global Const $olICal = 8 ; iCal format (.ics)
Global Const $olMHTML = 10 ; MIME HTML format (.mht)
Global Const $olMSG = 3 ; Outlook message format (.msg)
Global Const $olMSGUnicode = 9 ; Outlook Unicode message format (.msg)
Global Const $olRTF = 1 ; Rich Text format (.rtf)
Global Const $olTemplate = 2 ; Microsoft Office Outlook template (.oft)
Global Const $olTXT = 0 ; Text format (.txt)
Global Const $olVCal = 7 ; VCal format (.vcs)
Global Const $olVCard = 6 ; VCard format (.vcf)

; OlSensitivity Enumeration. Represents the sensitivity for the Microsoft Office Outlook item.
Global Const $olConfidential = 3 ; Confidential
Global Const $olNormal = 0 ; Normal sensitivity
Global Const $olPersonal = 1 ; Personal
Global Const $olPrivate = 2 ; Private

; OlShowItemCount Enumeration. Indicates which type of Microsoft Outlook item count is displayed for folders in the Outlook navigation pane.
Global Const $olNoItemCount = 0 ; No item count displayed
Global Const $olShowTotalItemCount = 2 ; Shows count of total number of items
Global Const $olShowUnreadItemCount = 1 ; Shows count of unread items

; OlStorageIdentifierType Enumeration. Specifies the type of identifier for a StorageItem object.
Global Const $olIdentifyByEntryID = 1 ; Identifies a StorageItem by EntryID
Global Const $olIdentifyByMessageClass = 2 ; Identifies a StorageItem by message class
Global Const $olIdentifyBySubject = 0 ; Identifies a StorageItem by Subject

; OlStoreType Enumeration. Indicates the format in which the data file should be created.
Global Const $olStoreANSI = 3 ; ANSI format personal folders file (.pst) compatible with all previous versions of Microsoft Office Outlook format
Global Const $olStoreDefault = 1 ; Default format compatible with the mailbox mode in which Microsoft Office Outlook runs on the Microsoft Exchange Server
Global Const $olStoreUnicode = 2 ; Unicode format personal folders file (.pst) compatible with Microsoft Office Outlook 2003 and later

; OlTaskRecipientType Enumeration. Indicates the task recipient type.
Global Const $olFinalStatus = 3 ; Indicates that the recipient will receive completion reports for the task
Global Const $olUpdate = 2 ; Indicates that the recipient will receive status updates for the task

; OlTaskResponse Enumeration. Indicates the response to a task request.
Global Const $olTaskAccept = 2 ; Task accepted
Global Const $olTaskAssign = 1 ; Task reassigned
Global Const $olTaskDecline = 3 ; Task declined
Global Const $olTaskSimple = 0 ; Task is a simple task and cannot be accepted, declined, or assigned. Not a valid parameter to the TaskItem.Respond method

; OlTrackingStatus. Indicates the most recent tracking status change for the recipient.
Global Const $olTrackingDelivered = 1 ; The item has been delivered to the recipient.
Global Const $olTrackingNone = 0 ; No tracking information is available for the recipient.
Global Const $olTrackingNotDelivered = 2 ; The item has not been delivered to the recipient.
Global Const $olTrackingNotRead = 3 ;The item has not been read by the recipient.
Global Const $olTrackingRead = 6 ; The item has been read by the recipient.
Global Const $olTrackingRecallFailure = 4 ; The sender of the item attempted to recall the item but was unsuccessful.
Global Const $olTrackingRecallSuccess = 5 ; The sender of the item recalled the item.
Global Const $olTrackingReplied = 7 ; The recipient replied to the item.

; OlUserPropertyType Enumeration. Indicates the user property type.
Global Const $olCombination = 19 ; The property type is a combination of other types
Global Const $olCurrency = 14 ; Represents a Currency property type
Global Const $olDateTime = 5 ; Represents a DateTime property type
Global Const $olDuration = 7 ; Represents a time duration property type
Global Const $olEnumeration = 21 ; Represents an enumeration property type
Global Const $olFormula = 18 ; Represents a formula property type. See UserDefinedProperty.Formula property
Global Const $olInteger = 20 ; Represents an Integer number property type
Global Const $olKeywords = 11 ; Represents a String array property type used to store keywords
Global Const $olNumber = 3 ; Represents a Double number property type
Global Const $olOutlookInternal = 0 ; Represents an Outlook internal property type.
Global Const $olPercent = 12 ; Represents a Double number property type used to store a percentage
Global Const $olSmartFrom = 22 ; Represents a smart from property type
Global Const $olText = 1 ; Represents a String property type
Global Const $olYesNo = 6 ; Represents a yes/no (Boolean) property type

; OlViewSaveOption enumeration. Specifies the folders in which the view is available and the read permissions attached to the view
Global Const $olViewSaveOptionAllFoldersOfType = 2 ; Indicates that the view is available in all folders of the same type
Global Const $olViewSaveOptionThisFolderEveryone = 0 ; Indicates that the view is only available in the current folder and is available to all users
Global Const $olViewSaveOptionThisFolderOnlyMe = 1 ; Indicates that the view is only available in the current folder and is only available to the current Outlook user

; OlViewType enumeration. Indicates the view object type
Global Const $olBusinessCardView = 5 ; Represents a BusinessCardView object
Global Const $olCalendarView = 2 ; Represents a CalendarView object
Global Const $olCardView = 1 ; Represents a CardView object
Global Const $olDailyTaskListView = 6 ; Represents the TableView object that contains the daily task list in a calendar view
Global Const $olIconView = 3 ; Represents an IconView object
Global Const $olPeopleView = 7 ; Represents a PeopleView object
Global Const $olTableView = 0 ; Represents a TableView object
Global Const $olTimelineView = 4 ; Represents a TimelineView object

; OlWindowState Enumeration. Indicates the window state.
Global Const $olMaximized = 0 ; The window is maximized
Global Const $olMinimized = 1 ; The window is minimized
Global Const $olNormalWindow = 2 ; The window is in the normal state (not minimized or maximized)
; ===============================================================================================================================
#EndRegion #CONSTANTS#

#Region #PROPERTY TAGS#
; >>> Detailed List - https://interoperability.blob.core.windows.net/files/MS-OXPROPS/%5bMS-OXPROPS%5d-190319.pdf <<<
; MAPI Properties
; https://docs.microsoft.com/en-us/office/client-developer/outlook/mapi/mapi-canonical-properties
; https://docs.microsoft.com/en-us/office/client-developer/outlook/mapi/mapi-reference
Global Const $sProptagURL = "http://schemas.microsoft.com/mapi/proptag/"

; Folder/Item Accesproperties
; https://docs.microsoft.com/en-us/office/client-developer/outlook/mapi/pidtagaccess-canonical-property and following
Global Const $sPR_ACCESS = $sProptagURL & "0x0FF40003"
Global Const $sPR_ACCESS_LEVEL = $sProptagURL & "0x0FF70003"
Global Const $sPR_ACL_TABLE = $sProptagURL & "0x0FF70003"

; Folder AutoArchive properties
; https://docs.microsoft.com/en-us/office/vba/outlook/how-to/solution-storage/save-auto-archive-properties-of-a-folder-in-solution-storage
Global Const $sPR_AGING_AGE_FOLDER = $sProptagURL & "0x6857000B"
Global Const $sPR_AGING_PERIOD = $sProptagURL & "0x36EC0003"
Global Const $sPR_AGING_GRANULARITY = $sProptagURL & "0x36EE0003"
Global Const $sPR_AGING_DELETE_ITEMS = $sProptagURL & "0x6855000B"
Global Const $sPR_AGING_FILE_NAME_AFTER9 = $sProptagURL & "0x6859001E"
Global Const $sPR_AGING_DEFAULT = $sProptagURL & "0x685E0003"

; Folder default message class
; https://docs.microsoft.com/en-us/office/client-developer/outlook/mapi/pidtagdefaultpostmessageclass-canonical-property
Global Const $sPR_DEF_POST_MSGCLASS = $sProptagURL & "0x36E5001E"
Global Const $sPR_DEF_POST_DISPLAYNAME = $sProptagURL & "0x36E6001E"

; Mail header
Global Const $sPR_MAIL_HEADER_TAG = $sProptagURL & "0x007D001E"

; Sizes of all properties on a message object, quotas and out-of-office status.
; https://docs.microsoft.com/en-us/office/client-developer/outlook/mapi/pidtagmessagesizeextended-canonical-property
Global Const $sPR_MESSAGE_SIZE_EXTENDED = $sProptagURL & "0x0E080014"
Global Const $sPR_QUOTA_WARNING = $sProptagURL & "0x341A0003"
Global Const $sPR_QUOTA_SEND = $sProptagURL & "0x341B0003"
Global Const $sPR_QUOTA_RECEIVE = $sProptagURL & "0x341C0003"
Global Const $sPR_MAX_SUBMIT_MESSAGE_SIZE = $sProptagURL & "0x666d0003"
Global Const $sPR_OOF_STAT = $sProptagURL & "0x661D000B"
#EndRegion #PROPERTY TAGS#

; #CURRENT# =====================================================================================================================
;_OL_Open
;_OL_Close
;_OL_ErrorNotify
;_OL_TestEnvironmentCreate
;_OL_VersionInfo
; ===============================================================================================================================

; #INTERNAL_USE_ONLY# ===========================================================================================================
;__OL_ArrayConcatenate
;__OL_CheckProperties
;__OL_Property2Hex
;__OL_PSTConvertUNC
;__Outlook_ErrorHandler
;__ParseCSV
;__WriteCSV
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Name...........: _OL_Open
; Description ...: Opens a connection to Microsoft Outlook.
; Syntax.........: _OL_Open([$bWarningClick = False[, $sWarningProgram = ""[, $iWinCheckTime = 1000[, $iCtrlCheckTime = 1000[, $sProfileName = ""[, $sPassword = ""]]]]])
; Parameters ....: $bWarningClick      - [optional] If set to True a function will click away the Outlook security warning messages (default = False)
;                  $sWarningProgram    - [optional] Complete path to the compiled WarningProgram, e.g.: "C:\OLext\_OL_Warnings.exe"
;                  |(default = @ScriptDir & "\_OL_Warnings.exe" which is part of this UDF)
;                  $iWinCheckTime      - [optional] Time in milliseconds to wait before a check for a warning window is performed (default = 1000 milliseconds = 1 second)
;                  $iCtrlCheckTime     - [optional] How long, in milliseconds, we will wait before we check that the controls we click are enabled (default = 1000)
;                  $sProfileName       - [optional] Name of the profile to be used for logon. The default profile will be used if none is specified (default = "")
;                  $sPassword          - [optional] The password associated with the profile (default = "")
; Return values .: Success - New object identifier and sets @extended:
;                  |0 - Outlook was not running. When using _OL_Open you need not to set parameter $bForceClose to True to stop the Outlook process (optional)
;                  |1 - Outlook was already up and running. When using _OL_Open you have to set parameter $bForceClose to True to stop the Outlook process (optional)
;                  Failure - Returns 0 and sets @error:
;                  |1  - Unable to create Outlook Object. @extended is set to the COM error code
;                  |3  - $bWarningClick is invalid. Has to be True or False
;                  |4  - $iWinCheckTime is invalid. Has to be an integer
;                  |5  - File $sWarningProgram does not exist. Has to be an executable that can be startet using the run command
;                  |6  - Run error when running $sWarningProgram. Please check @extended for more information
;                  |7  - $iCtrlCheckTime is invalid. Has to be an integer
;                  |8  - Unable to logon to the default profile. @extended is set to the COM error code
;                  |9  - No profile has been configured or there is no default profile for Outlook
;                  |10 - You specified a profile name to logon to but Outlook is already running
; Author ........: Wooltown
; Modified ......: water
; Remarks .......: Create your own $sWarningProgram Exe if the window title or messages have changed or are displayed in another language.
;                  If Outlook is already running a specified profile name is ignored because no logon is needed.
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _OL_Open($bWarningClick = False, $sWarningProgram = "", $iWinCheckTime = 1000, $iCtrlCheckTime = 1000, $sProfileName = "", $sPassword = "")
	If $bWarningClick = Default Then $bWarningClick = False
	If $sWarningProgram = Default Then $sWarningProgram = ""
	If $iWinCheckTime = Default Then $iWinCheckTime = 1000
	If $iCtrlCheckTime = Default Then $iCtrlCheckTime = 1000
	If $sProfileName = Default Then $sProfileName = ""
	If $sPassword = Default Then $sPassword = ""
	Local $oOL = ObjGet("", "Outlook.Application")
	If IsObj($oOL) Then $__bOL_AlreadyRunning = True
	If Not IsBool($bWarningClick) Then Return SetError(3, 0, 0)
	If Not IsInt($iWinCheckTime) Then Return SetError(4, 0, 0)
	If Not IsInt($iCtrlCheckTime) Then Return SetError(7, 0, 0)
	If $__bOL_AlreadyRunning And $sProfileName <> "" Then Return SetError(10, 0, 0)
	; Activate the COM error handler for older AutoIt versions
	If $__iOL_Debug = 0 And (Number(StringReplace(@AutoItVersion, ".", "")) < 3392 Or Number(StringReplace(@AutoItVersion, ".", "")) > 33120) Then
		_OL_ErrorNotify(4)
		SetError(0) ; Reset @error which is returned by _OL_ErrorNotify if a COM error handler has already been set up by the user
	EndIf
	If Not $__bOL_AlreadyRunning Then
		$oOL = ObjCreate("Outlook.Application")
		If @error Or Not IsObj($oOL) Then Return SetError(1, @error, 0)
	EndIf
	If $bWarningClick Then
		If StringStripWS($sWarningProgram, BitOr($STR_STRIPLEADING, $STR_STRIPTRAILING)) = "" Then $sWarningProgram = @ScriptDir & "\_OL_Warnings.exe"
		If Not FileExists($sWarningProgram) Then Return SetError(5, 0, 0)
		Run('"' & $sWarningProgram & '" ' & @AutoItPID & ' ' & $iWinCheckTime & ' ' & $iCtrlCheckTime & ' ' & $oOL.Version & ' ' & $oOL.LanguageSettings.LanguageID($msoLanguageIDUI), '', @SW_HIDE)
		If @error Then Return SetError(6, @error, 0)
	EndIf
	Local $aVersion = StringSplit($oOL.Version, '.')
	; Logon to the specified or the default profile if Outlook wasn't already running (for Outlook 2007 and later)
	If Not $__bOL_AlreadyRunning And Int($aVersion[1]) >= 12 Then
		If StringStripWS($sProfileName, BitOr($STR_STRIPLEADING, $STR_STRIPTRAILING)) = "" Then $sProfileName = $oOL.DefaultProfileName
		If StringStripWS($sProfileName, BitOr($STR_STRIPLEADING, $STR_STRIPTRAILING)) = "" Then Return SetError(9, 0, 0)
		Local $oNamespace = $oOL.GetNamespace("MAPI")
		$oNamespace.Logon($sProfileName, $sPassword, False, False)
		If @error Then Return SetError(8, @error, 0)
	EndIf
	If $__bOL_AlreadyRunning Then Return SetError(0, 1, $oOL)
	Return $oOL
EndFunc   ;==>_OL_Open

; #FUNCTION# ====================================================================================================================
; Name...........: _OL_Close
; Description ...: Closes the connection to Microsoft Outlook.
; Syntax.........: _OL_Close($oOL[, $bForceClose = False])
; Parameters ....: $oOL         - Outlook object returned by a preceding call to _OL_Open()
;                  $bForceClose - [optional] If True Outlook is closed even when it was running at _OL_Open time (default = False)
; Return values .: Success - Returns 1 and sets @extended to 1 if Outlook was already running
;                  Failure - Returns 0 and sets @error
;                  |1 - Error with Session.Logoff. @extended is set to the COM error code
;                  |2 - Error with Application.Quit. @extended is set to the COM error code
; Author ........: Wooltown
; Modified ......: water
; Remarks .......: If Outlook was already running when _OL_Open was called you have to use flag $bForceClose to close Outlook.
;                  If Outlook was already running when _OL_Open was called @extended is set to 1 to indicate this
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _OL_Close(ByRef $oOL, $bForceClose = False)
	If $bForceClose = Default Then $bForceClose = False
	If Not $__bOL_AlreadyRunning Or $bForceClose Then
		; Logs the user off from the current MAPI session.
		$oOL.Session.Logoff
		If @error Then Return SetError(1, @error, 0)
		; The associated Outlook session is closed completely.
		; The user is logged out of the messaging system and any changes to items not already saved are discarded
		$oOL.Application.Quit
		If @error Then Return SetError(2, @error, 0)
	EndIf
	$oOL = 0
	$__iOL_Debug = 0
	$__sOL_DebugFile = @ScriptDir & "\Outlook_Debug.txt"
	$__oOL_Error = 0
	If $__bOL_AlreadyRunning Then
		$__bOL_AlreadyRunning = False
		Return SetError(0, 1, 1)
	EndIf
	Return 1
EndFunc   ;==>_OL_Close

; #FUNCTION# ====================================================================================================================
; Name...........: _OL_ErrorNotify
; Description ...: Sets or queries the debug level.
; Syntax.........: _OL_ErrorNotify($iDebug[, $sDebugFile = @ScriptDir & "\Outlook_Debug.txt"])
; Parameters ....: $iDebug     - Debug level. Allowed values are:
;                  |-1 - Return the current settings
;                  |0  - Disable debugging
;                  |1  - Enable debugging. Output the debug info to the console
;                  |2  - Enable Debugging. Output the debug info to a MsgBox as $MB_ICONERROR
;                  |3  - Enable Debugging. Output the debug info to a file defined by $sDebugFile
;                  |4  - Enable Debugging. The COM errors will be handled (the script no longer crashes) without any output
;                  $sDebugFile - [optional] File to write the debugging info to if $iDebug = 3 (Default = @ScriptDir & "\Outlook_Debug.txt")
; Return values .: Success - The return value depends on the value of $iDebug:
;                  For $iDebug >= 0: Returns 1, sets @extended to:
;                  |0 - The COM error handler for this UDF was already active
;                  |1 - A COM error handler has been initialized for this UDF
;                  For $iDebug = -1: Returns a one based one-dimensional array with the following elements:
;                  |1 - Debug level. Value from 0 to 3. Check parameter $iDebug for details
;                  |2 - Debug file. File to write the debugging info to as defined by parameter $sDebugFile
;                  |3 - True if the COM error handler has been defined for this UDF. False if debugging is set off or a COM error handler was already defined
;                  Failure - 0, sets @error to:
;                  |1 - $iDebug is not an integer or < -1 or > 4
;                  |2 - Installation of the custom error handler failed. @extended is set to the error code returned by ObjEvent
;                  |3 - COM error handler already set to another function
; Author ........: water
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _OL_ErrorNotify($iDebug, $sDebugFile = "")
	If $sDebugFile = Default Then $sDebugFile = ""
	If Not IsInt($iDebug) Or $iDebug < -1 Or $iDebug > 4 Then Return SetError(1, 0, 0)
	If $sDebugFile = "" Then $sDebugFile = @ScriptDir & "\Outlook_Debug.txt"
	Switch $iDebug
		Case -1
			Local $avDebug[4] = [3]
			$avDebug[1] = $__iOL_Debug
			$avDebug[2] = $__sOL_DebugFile
			$avDebug[3] = IsObj($__oOL_Error)
			Return $avDebug
		Case 0
			$__iOL_Debug = 0
			$__sOL_DebugFile = ""
			$__oOL_Error = 0
		Case Else
			$__iOL_Debug = $iDebug
			$__sOL_DebugFile = $sDebugFile
			; A COM error handler will be initialized only if one does not exist
			If ObjEvent("AutoIt.Error") = "" Then
				$__oOL_Error = ObjEvent("AutoIt.Error", "__Outlook_ErrorHandler") ; Creates a custom error handler
				If @error <> 0 Then Return SetError(2, @error, 0)
				Return SetError(0, 1, 1)
			ElseIf ObjEvent("AutoIt.Error") = "__Outlook_ErrorHandler" Then
				Return SetError(0, 0, 1) ; COM error handler already set by a call to this function
			Else
				Return SetError(3, 0, 0) ; COM error handler already set to another function
			EndIf
	EndSwitch
	Return 1
EndFunc   ;==>_OL_ErrorNotify

; #FUNCTION# ====================================================================================================================
; Name ..........: _OL_TestEnvironmentCreate
; Description ...: Deletes and recreates the OutlookEX UDF test environment.
; Syntax.........: _OL_TestEnvironmentCreate($oOL[, $vDontAsk = ""[, $vDontDelete= ""]])
; Parameters ....: $oOL         - Outlook object returned by a preceding call to _OL_Open()
;                  $vDontAsk    - [optional] Return error if folder already exists? 1 = no, 4 = yes, "" = read value from ini file
;                  $vDontDelete - [optional] Delete test environment if it already exists? 1 = no, 4 = yes, "" = read value from ini file
; Return values .: Success - 1
;                  Failure - 0 and sets @error:
;                  |1 - Folder *\Outlook-UDF-Test already exists and user denied to delete/recreate the testenvironment
;                  |2 - Error deleting folder. @extended is @error as returned by _OL_FolderDelete
;                  |3 - Error creating root source folder (*\Outlook-UDF-Test\SourceFolder). @extended is set to @error as returned by _OL_FolderCreate
;                  |3xx - Error creating source folder. @extended is set to @error as returned by _OL_FolderCreate
;                  |4 - Error creating root target folder (*\Outlook-UDF-Test\TargetFolder). @extended is set to @error as returned by _OL_FolderCreate
;                  |4xx - Error creating target folder. @extended is set to @error as returned by _OL_FolderCreate
;                  |5xx - Error creating item in source folder. @extended is set to @error as returned by _OL_<itemtype>Create
;                  |6xx - Error creating item in target folder. @extended is set to @error as returned by _OL_<itemtype>Create
; Author ........: water
; Modified ......:
; Remarks .......: The test environment consists of the following items:
;                    * Folder Outlook-UDF-Test, subfolders plus items
;                    * Group "Outlook-UDF-Test" in the Outlook bar
;                    * Shortcut int the "Outlook-UDF-Test" group in the Outlook bar
;                    * Category "Outlook-UDF-Test" in the Outlook bar
;                    * Mail signature "Outlook-UDF-Test"
;                    * PST "Outlook-UDF-Test" in "C:\temp\Outlook-UDF-Test.pst"
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _OL_TestEnvironmentCreate($oOL, $vDontAsk = "", $vDontDelete = "")
	Local $vResult, $sCurrentUser = $oOL.GetNameSpace("MAPI").CurrentUser.Name
	#ignorefunc _OL_FolderExists, _OL_FolderDelete, _OL_FolderCreate
	#ignorefunc _OL_ItemCreate, _OL_ItemRecipientAdd, _OL_ItemRecurrenceSet, _OL_ItemRecurrenceExceptionSet, _OL_ItemAttachmentAdd, _OL_DistListMemberAdd
	#ignorefunc _DateAdd, _NowCalc, _NowCalcDate, _NowDate
	;---------------------------------
	; Delete existing folder structure
	;---------------------------------
	If _OL_FolderExists($oOL, "*\Outlook-UDF-Test") Then
		If StringStripWS($vDontAsk, BitOr($STR_STRIPLEADING, $STR_STRIPTRAILING)) = "" Then $vDontAsk = IniRead("_OL_TestEnvironment.ini", "Configuration", "DontAsk", "4") ; checked = 1, unchecked = 4
		If StringStripWS($vDontDelete, BitOr($STR_STRIPLEADING, $STR_STRIPTRAILING)) = "" Then $vDontDelete = IniRead("_OL_TestEnvironment.ini", "Configuration", "DontDelete", "4") ; checked = 1, unchecked = 4
		If $vDontDelete = 4 Then
			If $vDontAsk = 4 Then
				Local $iResult = MsgBox(BitOr($MB_ICONQUESTION, $MB_YESNOCANCEL), "OutlookEX UDF - Create Test Environment", "Testenvironment already exists. Should it be deleted and recreated?")
				If $iResult = 2 Then Return SetError(1, 0, 0)
				If $iResult = 7 Then Return 1
			EndIf
			If $vDontAsk = 1 Or $iResult = 6 Then _OL_FolderDelete($oOL, "*\Outlook-UDF-Test")
		Else
			Return 1
		EndIf
		If @error Then Return SetError(2, @error, 0)
	EndIf
	;------------------------------------------------------------
	; Create Source Folder plus one subfolder for every item type
	;------------------------------------------------------------
	_OL_FolderCreate($oOL, "Outlook-UDF-Test\SourceFolder", $olFolderInbox)
	If @error Then Return SetError(3, @error, 0)
	Local $oSourceFolderCalendar = _OL_FolderCreate($oOL, "Calendar", $olFolderCalendar, "*\Outlook-UDF-Test\SourceFolder")
	If @error Then Return SetError(300, @error, 0)
	Local $oSourceFolderContact = _OL_FolderCreate($oOL, "Contacts", $olFolderContacts, "*\Outlook-UDF-Test\SourceFolder")
	If @error Then Return SetError(301, @error, 0)
	; Mark the folder as address book and change the name
	$oSourceFolderContact.ShowAsOutlookAB = True
	$oSourceFolderContact.AddressBookName = "Outlook-UDF-Test"
	If @error Then Return SetError(302, @error, 0)
	Local $oSourceFolderMail = _OL_FolderCreate($oOL, "Mail", $olFolderInbox, "*\Outlook-UDF-Test\SourceFolder")
	If @error Then Return SetError(303, @error, 0)
	Local $oSourceFolderNotes = _OL_FolderCreate($oOL, "Notes", $olFolderNotes, "*\Outlook-UDF-Test\SourceFolder")
	If @error Then Return SetError(304, @error, 0)
	Local $oSourceFolderTasks = _OL_FolderCreate($oOL, "Tasks", $olFolderTasks, "*\Outlook-UDF-Test\SourceFolder")
	If @error Then Return SetError(305, @error, 0)
	;-----------------------------------
	; Create test items in Source Folder
	;-----------------------------------
	; Appointment
	Local $sStartTime = StringMid(_DateAdd("n", -10, _NowCalc()), 12, 5)
	Local $sEndTime = StringMid(_DateAdd("h", 3, _NowCalc()), 12, 5)
	Local $oItem = _OL_ItemCreate($oOL, $olAppointmentItem, $oSourceFolderCalendar, "", "Subject=TestAppointment", _
			"Start=" & _NowCalcDate() & " " & $sStartTime, "End=" & _NowCalcDate() & " " & $sEndTime, _
			"ReminderMinutesBeforeStart=15", "ReminderSet=True", "Location=Building A, Room 10")
	If @error Then Return SetError(500, @error, 0)
	; Appointment: Add optional recipient
	_OL_ItemRecipientAdd($oOL, $oItem, Default, $olOptional, $sCurrentUser)
	If @error Then Return SetError(501, @error, 0)
	; Appointment: Add recurrence: Daily with defined start and end date/time
	_OL_ItemRecurrenceSet($oOL, $oItem, Default, _NowCalcDate(), $sStartTime, _DateAdd("D", 14, _NowCalcDate()), $sEndTime, $olRecursDaily, "", "", "")
	If @error Then Return SetError(502, @error, 0)
	; Define exception
	Local $sTemp = _DateAdd("D", 1, _NowCalcDate())
	_OL_ItemRecurrenceExceptionSet($oOL, $oItem, Default, $sTemp & " " & $sStartTime & ":00", $sTemp & " 08:00:00", $sTemp & " 14:00:00", "TestException", "ExceptionBody")
	If @error Then Return SetError(503, @error, 0)
	;	; Appointment: Create a conflict
	;	$oItem = _OL_ItemCreate($oOL, $olAppointmentItem, $oSourceFolderCalendar, "", "Subject=TestAppointment-Conflict", _
	;			"Start=" & _NowCalcDate() & " " & $sStartTime, "End=" & _NowCalcDate() & " " & $sEndTime)
	;	If @error Then Return SetError(504, @error, 0)
	; Contact
	_OL_ItemCreate($oOL, $olContactItem, $oSourceFolderContact, "", "FirstName=TestFirstName", "LastName=TestLastName")
	If @error Then Return SetError(505, @error, 0)
	; Distribution List + Member
	$vResult = _OL_ItemCreate($oOL, $olDistributionListItem, $oSourceFolderContact, "", "Subject=TestDistributionList", "Importance=" & $olImportanceHigh)
	If @error Then Return SetError(506, @error, 0)
	_OL_DistListMemberAdd($oOL, $vResult, Default, $sCurrentUser)
	If @error Then Return SetError(507, @error, 0)
	; Mail plus attachment
	$vResult = _OL_ItemCreate($oOL, $olMailItem, $oSourceFolderMail, "", "Subject=TestMail", "BodyFormat=" & $olFormatHTML, "HTMLBody=Bodytext in <b>bold</b>.", "To=" & $sCurrentUser)
	If @error Then Return SetError(508, @error, 0)
	_OL_ItemAttachmentAdd($oOL, $vResult, Default, @ScriptDir & "\The_Outlook.jpg")
	If @error Then Return SetError(509, @error, 0)
	; Note
	_OL_ItemCreate($oOL, $olNoteItem, $oSourceFolderNotes, "", "Width=350", "Body=TestNote")
	If @error Then Return SetError(510, @error, 0)
	; Task
	_OL_ItemCreate($oOL, $olTaskItem, $oSourceFolderTasks, "", "Subject=TestSubject", "StartDate=" & _NowDate())
	If @error Then Return SetError(511, @error, 0)
	;------------------------------------------------------------
	; Create Target Folder plus one subfolder for every item type
	;------------------------------------------------------------
	_OL_FolderCreate($oOL, "TargetFolder", $olFolderInbox, "*\Outlook-UDF-Test")
	If @error Then Return SetError(4, @error, 0)
	_OL_FolderCreate($oOL, "Calendar", $olFolderCalendar, "*\Outlook-UDF-Test\TargetFolder")
	If @error Then Return SetError(400, @error, 0)
	_OL_FolderCreate($oOL, "Contacts", $olFolderContacts, "*\Outlook-UDF-Test\TargetFolder")
	If @error Then Return SetError(401, @error, 0)
	_OL_FolderCreate($oOL, "Mail", $olFolderInbox, "*\Outlook-UDF-Test\TargetFolder")
	If @error Then Return SetError(402, @error, 0)
	_OL_FolderCreate($oOL, "Notes", $olFolderNotes, "*\Outlook-UDF-Test\TargetFolder")
	If @error Then Return SetError(403, @error, 0)
	_OL_FolderCreate($oOL, "Tasks", $olFolderTasks, "*\Outlook-UDF-Test\TargetFolder")
	If @error Then Return SetError(404, @error, 0)
	Return 1
EndFunc   ;==>_OL_TestEnvironmentCreate

; #FUNCTION# ====================================================================================================================
; Name...........: _OL_VersionInfo
; Description ...: Returns a string holding version/date of OutlookEX_Base, OutlookEX and OutlookEX_GUI.
; Syntax.........: _OL_VersionInfo()
; Parameters ....: None
; Return values .: Success - string holding version/date of OutlookEX_Base, OutlookEX and OutlookEX_GUI separated by @CRLF.
; Author ........: water
; Modified.......:
; Remarks .......: Example of the returned string: "OutlookEX_Base: 1.2.2.0 2020-09-01 <@CRLF> OutlookEX: 1.6.3.0 2020-09-01"
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _OL_VersionInfo()
	Local $sResult = "  " & $__g_sVersionOutlookEX_Base
	#forcedef $__g_sVersionOutlookEX, $__g_sVersionOutlookEX_GUI
	If IsDeclared("__g_sVersionOutlookEX") Then $sResult &= @CRLF & "  " & $__g_sVersionOutlookEX
	If IsDeclared("__g_sVersionOutlookEX_GUI") Then $sResult &= @CRLF & "  " & $__g_sVersionOutlookEX_GUI
	Return $sResult
EndFunc   ;==>_OL_VersionInfo

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: __OL_ArrayConcatenate
; Description ...: Concatenates 2D arrays.
; Syntax.........: __OL_ArrayConcatenate(ByRef $avArrayTarget, Const ByRef $avArraySource)
; Parameters ....: $avArrayTarget - The array to concatenate onto
;                  $avArraySource - The array to concatenate from - Must be 1D or 2D to match $avArrayTarget,
;                                   and if 2D, then Ubound($avArraySource, 2) <= Ubound($avArrayTarget, 2).
;                  $iFlags        - Flags as defined in function call for _OL_ItemFind
; Return values .: Success - Index of last added item
;                  Failure - -1, sets @error to 1 and @extended per failure (see code below)
; Author ........: Ultima
; Modified.......: PsaltyDS - 1D/2D version, changed return value and @error/@extended to be consistent with __ArrayAdd()
;                  water - removed 1D array support, support for row 1 containing the number of rows/columns
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func __OL_ArrayConcatenate(ByRef $avArrayTarget, Const ByRef $avArraySource, $iFlags)
	If Not IsArray($avArrayTarget) Then Return SetError(1, 1, -1) ; $avArrayTarget is not an array
	If Not IsArray($avArraySource) Then Return SetError(1, 2, -1) ; $avArraySource is not an array
	Local $iUBoundTarget0 = UBound($avArrayTarget, 0), $iUBoundSource0 = UBound($avArraySource, 0)
	If $iUBoundTarget0 <> $iUBoundSource0 Then Return SetError(1, 3, -1) ; 1D/2D dimensionality did not match
	If $iUBoundTarget0 > 2 Then Return SetError(1, 4, -1) ; At least one array was 3D or more
	Local $iUBoundTarget1 = UBound($avArrayTarget, 1), $iUBoundSource1 = UBound($avArraySource, 1)
	Local $iNewSize = $iUBoundTarget1 + $iUBoundSource1 - 1
	Local $iUBoundTarget2 = UBound($avArrayTarget, 2), $iUBoundSource2 = UBound($avArraySource, 2)
	If $iUBoundSource2 > $iUBoundTarget2 Then Return SetError(1, 5, -1) ; 2D boundry of source too large for target
	ReDim $avArrayTarget[$iNewSize][$iUBoundTarget2]
	For $r = 1 To $iUBoundSource1 - 1
		For $c = 0 To $iUBoundSource2 - 1
			$avArrayTarget[$iUBoundTarget1 + $r - 1][$c] = $avArraySource[$r][$c]
		Next
	Next
	If BitAND($iFlags, 2) <> 2 Then
		$avArrayTarget[0][0] = $iNewSize - 1
		If UBound($avArrayTarget, 0) > 1 And UBound($avArrayTarget, 2) > 1 Then $avArrayTarget[0][1] = UBound($avArrayTarget, 2)
	EndIf
	Return $iNewSize - 1
EndFunc   ;==>__OL_ArrayConcatenate

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: __OL_CheckProperties
; Description ...: Checks if specified item or user properties exist for the item and have the correct case.
; Syntax.........: __OL_CheckProperties($oItem, $aProperties[, $iFlag = 0[, $bItemProperties = True]])
; Parameters ....: $oItem           - Object of the item to check
;                  $aProperties     - Zero based array of property names. Format "propertyname=propertyvalue" is valid as well
;                  $iFlag           - [optional] 0: Array $aProperties is zero based, 1: Array $aProperties is one based (default = 0)
;                  $bItemProperties - Check item or user proferties (default = True = item properties)
; Return values .: Success - 1
;                  Failure - 0 and sets @error:
;                  |10nn - Property is not valid for this type of item. nn = index of property in error (zero or one based depends on parameter $iFlag)
;                  |11nn - Property has wrong case (e.g. "firstname" is wrong, "FirstName" is correct). nn = index of property in error (zero or one based depends on parameter $iFlag)
; Author ........: water
; Modified.......:
; Remarks .......: All pseudo properties (they start with @ like @ItemObject and @FolderObject) are not checked.
;                  So "invalid" pseudo properties will not raise an error. They will simply be ignored by the function who calls __OL_CheckProperties.
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func __OL_CheckProperties($oItem, $aProperties, $iFlag = 0, $bItemProperties = True)
	Local $aTemp, $iIndex, $iEnd, $oProperty, $bUserProperties = True
	If $iFlag = 1 Then
		$iIndex = 1
		$iEnd = $aProperties[0]
	Else
		$iIndex = 0
		$iEnd = UBound($aProperties) - 1
	EndIf
	If $oItem.Class = $olNote Then $bUserProperties = False ; Note items only support the ItemProperties property
	For $iIndex = $iIndex To $iEnd
		$aTemp = StringSplit($aProperties[$iIndex], "=")
		; Ignore pseudo properties starting with @
		If Stringleft($aTemp[1], 1) = "@" Then ContinueLoop
		If $aTemp[1] <> "" Then
			SetError(0)
			If $bUserProperties Then
				$oProperty = $oItem.UserProperties.Find($aTemp[1], Not $bItemProperties) ; Search Itemproperties or UserProperties
			Else
				$oProperty = $oItem.ItemProperties() ; Get the property collection
				$oProperty = $oProperty.Item($aTemp[1]) ; Get the named property
				If Not IsObj($oProperty) Then SetError(1) ; Set error if property not found
			EndIf
			If @error Then Return SetError(Int(10 & StringRight("0" & $iIndex, 2)), 0, 0) ; Property not found
			; Wrong case for EntryID does not get caught by the above code
			If ($aTemp[1] = "EntryID" And Not($aTemp[1] == "EntryID")) Then Return SetError(Int(11 & StringRight("0" & $iIndex, 2)), 0, 0)
			; Case doesn't match for everything else
			If Not($aTemp[1] == $oProperty.Name) Then Return SetError(Int(11 & StringRight("0" & $iIndex, 2)), 0, 0) ; Case doesn't match
		EndIf
	Next
	Return 1
EndFunc   ;==>__OL_CheckProperties

Func __OL_CheckProperties_OLD($oItem, $aProperties, $iFlag = 0)
	Local $sItemProperties = ",", $aTemp, $iIndex, $iEnd
	For $oProperty In $oItem.ItemProperties
		$sItemProperties = $sItemProperties & $oProperty.Name & ","
	Next
	If $iFlag = 1 Then
		$iIndex = 1
		$iEnd = $aProperties[0]
	Else
		$iIndex = 0
		$iEnd = UBound($aProperties) - 1
	EndIf
	For $iIndex = $iIndex To $iEnd
		$aTemp = StringSplit($aProperties[$iIndex], "=")
		If $aTemp[1] <> "" And StringInStr($sItemProperties, "," & $aTemp[1] & ",", $STR_NOCASESENSE) = 0 Then Return SetError(100, $iIndex, 0)
		If $aTemp[1] <> "" And StringInStr($sItemProperties, "," & $aTemp[1] & ",", $STR_CASESENSE) = 0 Then Return SetError(101, $iIndex, 0)
	Next
	Return 1
EndFunc   ;==>__OL_CheckProperties_OLD

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __Outlook_ErrorHandler
; Description ...: Called if an ObjEvent error occurs.
; Syntax.........: __Outlook_ErrorHandler()
; Parameters ....: None
; Return values .: @error is set to the COM error by AutoIt
; Author ........: water
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func __Outlook_ErrorHandler()
	Local $sError = "COM Error Encountered in " & @ScriptName & @CRLF & _
			"UDF version = " & @CRLF & _OL_VersionInfo() & @CRLF & _
			"@AutoItVersion = " & @AutoItVersion & @CRLF & _
			"@AutoItX64 = " & @AutoItX64 & @CRLF & _
			"@Compiled = " & @Compiled & @CRLF & _
			"@OSArch = " & @OSArch & @CRLF & _
			"@OSVersion = " & @OSVersion & @CRLF & _
			"Scriptline = " & $__oOL_Error.Scriptline & @CRLF & _
			"NumberHex = 0x" & Hex($__oOL_Error.Number, 8) & @CRLF & _
			"Number = " & $__oOL_Error.Number & @CRLF & _
			"WinDescription = " & StringStripWS($__oOL_Error.WinDescription, $STR_STRIPTRAILING) & @CRLF & _
			"Description = " & StringStripWS($__oOL_Error.Description, $STR_STRIPTRAILING) & @CRLF & _
			"Source = " & $__oOL_Error.Source & @CRLF & _
			"HelpFile = " & $__oOL_Error.HelpFile & @CRLF & _
			"HelpContext = " & $__oOL_Error.HelpContext & @CRLF & _
			"LastDllError = " & $__oOL_Error.LastDllError
	If $__iOL_Debug > 0 Then
		If $__iOL_Debug = 1 Then ConsoleWrite($sError & @CRLF & "========================================================" & @CRLF)
		If $__iOL_Debug = 2 Then MsgBox($MB_ICONERROR, "OutlookEX_Base UDF - Debug Info", $sError)
		If $__iOL_Debug = 3 Then FileWrite($__sOL_DebugFile, @YEAR & "." & @MON & "." & @MDAY & " " & @HOUR & ":" & @MIN & ":" & @SEC & " " & @CRLF & _
				"-------------------" & @CRLF & $sError & @CRLF & "========================================================" & @CRLF)
	EndIf
EndFunc   ;==>__Outlook_ErrorHandler

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: __ParseCSV
; Description ...: Reads a CSV-file
; Syntax.........: __ParseCSV($sFile, $sDelimiters=',', $sQuote='"', $iFormat=0)
; Parameters ....: $sFile       - File to read or string to parse
;                  $sDelimiters - [optional] Fieldseparators of CSV, multiple are allowed (default = ,;)
;                  $sQuote      - [optional] Character to quote strings (default = ")
;                  $iFormat     - [optional] Encoding of the file (default = 0):
;                  |-1     - No file, plain data given
;                  |0 or 1 - automatic (ASCII)
;                  |2      - Unicode UTF16 Little Endian reading
;                  |3      - Unicode UTF16 Big Endian reading
;                  |4 or 5 - Unicode UTF8 reading
; Return values .: Success - 2D-Array with CSV data (0-based)
;                  Failure - 0, sets @error to:
;                  |1 - could not open file
;                  |2 - error on parsing data
;                  |3 - wrong format chosen
; Author ........: ProgAndy
; Modified.......:
; Remarks .......:
; Related .......: __WriteCSV
; Link ..........: http://www.autoitscript.com/forum/topic/114406-csv-file-to-multidimensional-array
; Example .......:
; ===============================================================================================================================
Func __ParseCSV($sFile, $sDelimiters = ',;', $sQuote = '"', $iFormat = 0)
	Local Static $aEncoding[6] = [0, 0, 32, 64, 128, 256]
	If $iFormat < -1 Or $iFormat > 6 Then
		Return SetError(3, 0, 0)
	ElseIf $iFormat > -1 Then
		Local $hFile = FileOpen($sFile, $aEncoding[$iFormat])
		If @error Then Return SetError(1, @error, 0)
		$sFile = FileRead($hFile)
		FileClose($hFile)
	EndIf
	If $sDelimiters = "" Or IsKeyword($sDelimiters) Then $sDelimiters = ',;'
	If $sQuote = "" Or IsKeyword($sQuote) Then $sQuote = '"'
	$sQuote = StringLeft($sQuote, 1)
	Local $srDelimiters = StringRegExpReplace($sDelimiters, '[\\\^\-\[\]]', '\\\0')
	Local $srQuote = StringRegExpReplace($sQuote, '[\\\^\-\[\]]', '\\\0')
	Local $sPattern = StringReplace(StringReplace('(?m)(?:^|[,])\h*(["](?:[^"]|["]{2})*["]|[^,\r\n]*)(\v+)?', ',', $srDelimiters, 0, $STR_CASESENSE), '"', $srQuote, 0, $STR_CASESENSE)
	Local $aREgex = StringRegExp($sFile, $sPattern, $STR_REGEXPARRAYGLOBALMATCH)
	If @error Then Return SetError(2, @error, 0)
	$sFile = '' ; save memory
	Local $iBound = UBound($aREgex), $iIndex = 0, $iSubBound = 1, $iSub = 0
	Local $aResult[$iBound][$iSubBound]
	For $i = 0 To $iBound - 1
		Select
			Case StringLen($aREgex[$i]) < 3 And StringInStr(@CRLF, $aREgex[$i])
				$iIndex += 1
				$iSub = 0
				ContinueLoop
			Case StringLeft(StringStripWS($aREgex[$i], $STR_STRIPLEADING), 1) = $sQuote
				$aREgex[$i] = StringStripWS($aREgex[$i], BitOr($STR_STRIPLEADING, $STR_STRIPTRAILING))
				$aResult[$iIndex][$iSub] = StringReplace(StringMid($aREgex[$i], 2, StringLen($aREgex[$i]) - 2), $sQuote & $sQuote, $sQuote, 0, $STR_CASESENSE)
			Case Else
				$aResult[$iIndex][$iSub] = $aREgex[$i]
		EndSelect
		$aREgex[$i] = 0 ; save memory
		$iSub += 1
		If $iSub = $iSubBound Then
			$iSubBound += 1
			ReDim $aResult[$iBound][$iSubBound]
		EndIf
	Next
	If $iIndex = 0 Then $iIndex = 1
	ReDim $aResult[$iIndex][$iSubBound]
	Return $aResult
EndFunc   ;==>__ParseCSV

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: __WriteCSV
; Description ...: Writes a CSV-file (appends to an existing file)
; Syntax.........: __WriteCSV($sFile, Const ByRef $aData, $sDelimiter, $sQuote, $iFormat=0)
; Parameters ....: $sFile      - Destination file
;                  $aData      - [Const ByRef] 1-based 2D-Array with data
;                  $sDelimiter - [optional] Fieldseparator (default = ,)
;                  $sQuote     - [optional] Quote character (default = ")
;                  $iFormat    - [optional] character encoding of file (default = 0)
;                  |0 or 1 - ASCII writing
;                  |2      - Unicode UTF16 Little Endian writing (with BOM)
;                  |3      - Unicode UTF16 Big Endian writing (with BOM)
;                  |4      - Unicode UTF8 writing (with BOM)
;                  |5      - Unicode UTF8 writing (without BOM)
; Return values .: Success - Number of records written
;                  Failure - 0, sets @error to:
;                  |1 - No valid 2D-Array
;                  |2 - Could not open file
; Author ........: ProgAndy
; Modified.......: water
; Remarks .......:
; Related .......: __ParseCSV
; Link ..........: http://www.autoitscript.com/forum/topic/114406-csv-file-to-multidimensional-array
; Example .......:
; ===============================================================================================================================
Func __WriteCSV($sFile, Const ByRef $aData, $sDelimiter = ',', $sQuote = '"', $iFormat = 0)
	Local Static $aEncoding[6] = [9, 9, 41, 73, 137, 265] ; Mode + 1 (append to end of file) + 8 (create dir structure)
	If $sDelimiter = "" Or IsKeyword($sDelimiter) Then $sDelimiter = ','
	If $sQuote = "" Or IsKeyword($sQuote) Then $sQuote = '"'
	Local $iBound = UBound($aData, 1), $iSubBound = UBound($aData, 2)
	If Not $iSubBound Then Return SetError(1, 0, 0)
	Local $hFile = FileOpen($sFile, $aEncoding[$iFormat])
	If @error Then Return SetError(2, @error, 0)
	For $i = 1 To $iBound - 1
		For $j = 0 To $iSubBound - 1
			FileWrite($hFile, $sQuote & StringReplace($aData[$i][$j], $sQuote, $sQuote & $sQuote, 0, $STR_CASESENSE) & $sQuote)
			If $j < $iSubBound - 1 Then FileWrite($hFile, $sDelimiter)
		Next
		FileWrite($hFile, @CRLF)
	Next
	FileClose($hFile)
	Return $iBound - 1
EndFunc   ;==>__WriteCSV

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: __OL_Property2Hex
; Description ...: Translates MAPI property names to hex
; Syntax.........: __OL_Property2Hex($sProperty)
; Parameters ....: $sProperty - MAPI property name to translate
; Return values .: Success - Hex value of the specified property
;                  Failure - "", sets @error to:
;                  |1 - Property name could not be found in the table
; Author ........: water
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func __OL_Property2Hex($sProperty)
	Local $aProperties[100][2] = [[2, 2], ["subject", "0x0037001E"], ["SenderName", "0x0C1A001F"]]
	For $i = 1 To $aProperties[0][0]
		If $aProperties[$i][0] = $sProperty Then Return $aProperties[$i][1]
	Next
	Return SetError(1, 0, "")
EndFunc   ;==>__OL_Property2Hex

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: __OL_PSTConvertUNC
; Description ...: Translates PST filepath to UNC notation
; Syntax.........: __OL_PSTConvertUNC($sPSTPath)
; Parameters ....: $sPSTPath - Path of the PST file. Either in "X:" or "\\" notation. E.g.
;                  X:\Share\Archive.pst becomes \\?\UNC\Server\Share\Archive.pst or
;                  \\Server\Share\Archive.pst becomes \\?\UNC\Server\Share\Archive.pst
; Return values .: Success - To UNC notation converted path of the PST.
;                  If no conversion was needed $sPSTPath is returned unmodified and @extended is set to one of the following values:
;                  |1 - $sPSTPath was already in UNC notation
;                  |2 - $sPSTPath was passed in a notation which does not need to be converted.
;                  |    Does not start with "\\" or is not a network drive as returned by Drive GetType
;                  Failure - Unchanged $sPSTPath, sets @error to:
;                  |1 - Specified file does not exist
; Author ........: colombeen
; Modified.......: water
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func __OL_PSTConvertUNC($sPSTPath)
	Local $sPathTest
	If Not FileExists($sPSTPath) Then Return SetError(1, 0, $sPSTPath)
	If StringLeft($sPSTPath, 7) = "\\?\UNC" Then Return SetError(0, 1, $sPSTPath)
	$sPathTest = StringLeft($sPSTPath, 2)
	If $sPathTest = "\\" Then Return "\\?\UNC" & StringTrimLeft($sPSTPath, 1) ; UNC path
	If DriveGetType($sPathTest) = "Network" Then Return "\\?\UNC" & StringTrimLeft(DriveMapGet($sPathTest), 1) & StringTrimLeft($sPSTPath, 2) ; Mapped drive to UNC conversion
	Return SetError(0, 2, $sPSTPath) ; No conversion needed
EndFunc   ;==>__OL_PSTConvertUNC